
(* Analyse lexicale *)

{

  open Lexing
  open Parser
  open Ast

  (* Erreurs lexicales *)

  exception Lexical_error of string

  let id_or_keyword =
    let h = Hashtbl.create 17 in
    List.iter (fun (s,k) -> Hashtbl.add h s k)
      [ 
	"and", AND;
	"array", ARRAY;
	"begin", BEGIN;
	"boolean", BOOLEAN;
	"case", CASE;
	"div", DIV;
	"do", DO;
	"else", ELSE;
	"end", END;
	"false", FALSE;
	"for", FOR;
	"function", FUNCTION;
	"if", IF;
	"integer", INTEGER;
	"mod", MOD;
	"not", NOT;
	"of", OF;
	"or", OR;
	"procedure", PROCEDURE;
	"program", PROGRAM;
	"real", REAL;
	"then", THEN;
	"to", TO;
	"true", TRUE;
	"type", TYPE;
	"var", VAR;
	"while", WHILE;
      ];
    fun s -> 
      let s = String.lowercase s in
      try Hashtbl.find h s with Not_found -> IDENT s

  let newline lexbuf =
    let pos = lexbuf.lex_curr_p in
    lexbuf.lex_curr_p <- 
      { pos with pos_lnum = pos.pos_lnum + 1; pos_bol = pos.pos_cnum; 
        pos_cnum=0 }

}

let alpha = ['a'-'z' 'A'-'Z']
let digit = ['0'-'9']
let ident = alpha (alpha | '_' | digit)*

let exponent = ('e' | 'E') ('+' | '-')? digit+
let float = digit+ '.' digit+ exponent?
	  | digit+ exponent

rule token = parse
  | '\n' 
      { newline lexbuf; token lexbuf }
  | [' ' '\t' '\r']+ 
      { token lexbuf }
  | "(*"   
      { comment1 lexbuf; token lexbuf }
  | "{" 
      { comment2 lexbuf; token lexbuf }
  | ident  
      { id_or_keyword (lexeme lexbuf) }
  | digit+ 
      { CONST_INTEGER (int_of_string (lexeme lexbuf)) } 
  | float  
      { CONST_REAL (float_of_string (lexeme lexbuf)) } 
  | '('
      { LPAR }
  | ')'
      { RPAR }
  | '['
      { LSQUARE }
  | ']' 
      { RSQUARE }
  | '|'
      { BAR }
  | ':'
      { COLON }
  | ','
      { COMMA }
  | ';'
      { SEMICOLON }
  | '.'
      { DOT }
  | ".."
      { DOTDOT }
  | "-"
      { MINUS }
  | "+"
      { PLUS }
  | "*"
      { STAR }
  | "/"
      { SLASH }
  | "=" 
      { EQUAL }
  | ">" 
      { COMP Bgt }
  | ">=" 
      { COMP Bge }
  | "<" 
      { COMP Blt }
  | "<=" 
      { COMP Ble }
  | "<>"
      { COMP Bneq }
  | ":=" 
      { COLONEQUAL }
  | "_" 
      { UNDERSCORE }
  | _ 
      { raise (Lexical_error ("illegal character: " ^ lexeme lexbuf)) }
  | eof
      { EOF }

and comment1 = parse
  | "*)" { () }
  | '\n' { newline lexbuf; comment1 lexbuf }
  | _    { comment1 lexbuf }
  | eof  { raise (Lexical_error "unterminated comment") }

and comment2 = parse
  | "}"  { () }
  | '\n' { newline lexbuf; comment2 lexbuf }
  | _    { comment2 lexbuf }
  | eof  { raise (Lexical_error "unterminated comment") }

