
(* Exhaustivit du filtrage *)

open Ast

exception NotExhaustive

(* table des types rcursifs (car on a besoin du nombre de constructeurs) *)

let types = Hashtbl.create 17

let add_type ty = Hashtbl.add types ty.a_name ty

module Cmap = Map.Make(struct type t = constructor let compare = compare end)

let args = function Cconst _ -> [] | Cnamed c -> c.c_args

let is_var = function Pall | Pid _ -> true | _ -> false

(* indique si [n] est le nb de constructeurs de [ty] *)
let not_enough_for n = function
  | Tboolean -> n < 2
  | Tinteger | Treal -> true (* lger abus *)
  | Tarray _ -> assert false
  | Tid x ->
      let d = Hashtbl.find types x in
      n < List.length d.a_constructors

(* Dtection du caractre exhaustif *)

(* la matrice est reprsente par la liste de ses lignes *)

type row = (t_type * t_pattern) list
type matrix = row list

let exhaustive ty pl =
  let rec matrix = function
    | [] -> 
	(* aucune ligne *)
	raise NotExhaustive
    | [] :: _ -> 
	(* au moins une action : OK *)
	()
	(* note : s'il y en a plus, cela ne signifie pas ncessairement
	   que la clause est inutile car l'algo duplique des actions *)
    | (((ty,_) :: _) :: _) as m ->
	if List.for_all (fun r -> is_var (snd (List.hd r))) m then
	  matrix (List.map List.tl m)
	else
	  (* on spare les lignes par constructeurs *)
	  let cm0 = 
	    List.fold_left 
	      (fun cm p -> match p with
		 | (_, Pconstructor (c, _)) :: _ -> Cmap.add c [] cm
		 | (_, (Pall | Pid _)) :: _ -> cm
		 | [] -> assert false)
	      Cmap.empty m
	  in
	  let dispatch r cm = match r with
	    | [] -> 
		assert false
	    | (_, (Pall | Pid _)) :: rr ->
		(* une variable => on doit la reprendre pour chaque constr. *)
		Cmap.fold
		  (fun c m cm -> 
		     let h = List.map (fun ty -> ty, Pall) (args c) in
		     Cmap.add c ((h @ rr) :: m) cm)
		  cm cm
	    | (_, (Pconstructor (c, pl))) :: rr ->
		let h = List.combine (args c) pl in
		let m = Cmap.find c cm in
		Cmap.add c ((h @ rr) :: m) cm
	  in
	  let cm = List.fold_right dispatch m cm0 in
	  let variable r mr = match r with
	      | [] -> assert false
	      | (_, (Pall | Pid _)) :: rr -> rr :: mr
	      | (_, Pconstructor _) :: _ -> mr
	  in
	  let nbc = ref 0 in
	  Cmap.iter (fun _ m -> incr nbc; matrix m) cm;
	  if not_enough_for !nbc ty then
	    let mr = List.fold_right variable m [] in
	    matrix mr
  in
  matrix (List.map (fun (p,_) -> [ty, p]) pl)

